package org.capcaval.ermine.layout;

import java.util.ArrayList;
import java.util.List;

import javafx.geometry.Point2D;

public abstract class RefPointLayout<P, N> implements NodeLayout,
		NodeLayoutComputer {

	protected List<NodeWrapper<N>> wrapperList = new ArrayList<NodeWrapper<N>>();
	protected P pane;
	private int widthInPixel;
	private int heightInPixel;

	public RefPointLayout(P pane) {
		this.pane = pane;

		this.init(pane);
	}

	abstract protected void init(P pane);
	abstract protected void addNode(N node);
	abstract protected void removeNodeinParent(N node);
	abstract protected int getWidth(N node);
	abstract protected int getHeight(N node);
	
	

	public void addNode(N node, RefPointEnum refpoint) {
		// if null add it without layout
		if (refpoint != null) {
			// call the other addshape without deltax and deltay
			this.addNode(node, refpoint, 0, 0);
		} else {
			// add it to the shapePanel without a layout
			this.addNode(node);
		}
	}
	
	public void removeNode(N node){
		for(NodeWrapper<?>wrapper : this.wrapperList){
			if(wrapper.getPoint1().getNode() == node){
				// remove
				this.wrapperList.remove(node);
				// remove from parent
				this.removeNodeinParent(node);
				
				// get out
				break;
			}
		}
	}
	
	public void addNode(N node, RefPointEnum refPoint, int deltaX, int deltaY) {
		// wrap it
		NodeWrapper<N> wrapper = new NodeWrapper<N>(node, refPoint, deltaX,
				deltaY);
		// keep it
		this.wrapperList.add(wrapper);
		// add it to the shapePanel
		this.addNode(node);
		// compute the minimum size
		// TODO it is going to be easy
		
		// recompute with the current size
		this.computeLayout(this.widthInPixel, this.heightInPixel);
	}
	
	public void addCenteredNode(N node, RefPointEnum refPoint) {
		this.addCenteredNode(node, refPoint, 0, 0);
	}

	public void addCenteredNode(N node, RefPointEnum refPoint, int deltaX, int deltaY) {

		int width = this.getWidth(node);
		int height = this.getWidth(node);
		
		// wrap it
		NodeWrapper<N> wrapper = new NodeWrapper<N>(node, refPoint, deltaX-width/2,
				deltaY-height/2);
		// keep it
		this.wrapperList.add(wrapper);
		// add it to the shapePanel
		this.addNode(node);
		// compute the minimum size
		// TODO it is going to be easy
		
		// recompute with the current size
		this.computeLayout(this.widthInPixel, this.heightInPixel);
	}

	
	public void addResizableNode(N node, 			
			int tlDeltaX, int tlDeltaY, 
			int brDeltaX, int brDeltaY) {
		// wrap it
		NodeWrapper<N> wrapper = new NodeWrapper<N>(
				node,  RefPointEnum.topLeftPoint, tlDeltaX, tlDeltaY, RefPointEnum.bottomRightPoint, brDeltaX, brDeltaY);
		// keep it
		this.wrapperList.add(wrapper);
		// add it to the shapePanel
		this.addNode(node);
		
		// recompute with the current size
		this.computeLayout(this.widthInPixel, this.heightInPixel);
	}

	public void addResizableNode(N node,
			RefPointEnum tlRefpoint,
			int tlDeltaX, int tlDeltaY,
			RefPointEnum brRefpoint,
			int brDeltaX, int brDeltaY) {
		// wrap it
		NodeWrapper<N> wrapper = new NodeWrapper<N>(
				node,  tlRefpoint, tlDeltaX, tlDeltaY, brRefpoint, brDeltaX, brDeltaY);
		// keep it
		this.wrapperList.add(wrapper);
		// add it to the shapePanel
		this.addNode(node);
		
		// recompute with the current size
		this.computeLayout(this.widthInPixel, this.heightInPixel);
	}

	
//	public void addResizableWidthNode(N node, RefPointEnum refpoint){
//		
//	}
	
	@Override
	public void computeLayout(int newWidthInPixel, int newHeightInPixel) {
		this.computeLayout(this.wrapperList, newWidthInPixel, newHeightInPixel);

	}

	public void computeLayout(List<NodeWrapper<N>> shapeList,
			int newWidthInPixel, int newHeightInPixel) {
		// firstly keep the bound
		this.widthInPixel = newWidthInPixel;
		this.heightInPixel = newHeightInPixel;
		
		// secondly compute all the ref point
		ComputedRefPoint allRefPoint = this.computeNewRefPoints(
				newWidthInPixel, newHeightInPixel);

		// compute new position for the given shape
		for (NodeWrapper<N> wrapper : shapeList) {
			// one point ref
			if (wrapper.point2 == null) {
				// get the shape
				N node = wrapper.getPoint1().getNode();
				// get the ref point position
				Point2D point = allRefPoint.getPoint(wrapper.getPoint1().getRefPoint());
				// compute the new position
				this.setXOnNode(node, point.getX() + wrapper.getPoint1().getDeltaX());
				this.setYOnNode(node, point.getY() + wrapper.getPoint1().getDeltaY());
			}else{ // two point ref
				// get the topleft ref point position
				Point2D refPointTL = allRefPoint.getPoint(wrapper.point1.refPoint);
				// get the bottom rightref point position
				Point2D refPointBR = allRefPoint.getPoint(wrapper.point2.refPoint);

				Point2D pointTL = new Point2D(
						refPointTL.getX() + wrapper.getPoint1().getDeltaX(), 
						refPointTL.getY() + wrapper.getPoint1().getDeltaY()); 
				Point2D pointBR = new Point2D(
						refPointBR.getX() + wrapper.getPoint2().getDeltaX(), 
						refPointBR.getY() + wrapper.getPoint2().getDeltaY()); 
				
				// Compute the width of the node
				double width = pointBR.getX() - pointTL.getX(); 
				// Compute the height of the node
				double height = pointBR.getY() - pointTL.getY();
				
				N node = wrapper.getPoint1().getNode();

				// set width and height 
				this.setXOnNode(node, pointTL.getX());
				this.setYOnNode(node, pointTL.getY());

				// set width and height 
				this.setWidthOnNode(node, width);
				this.setHeightOnNode(node, height);
			}
		}
	}

	abstract protected void setXOnNode(N node, double d);

	abstract protected void setYOnNode(N node, double d);

	abstract protected void setWidthOnNode(N node, double d);

	abstract protected void setHeightOnNode(N node, double d);

	
	protected ComputedRefPoint computeNewRefPoints(int newWidthInPixel,
			int newHeightInPixel) {
		int top = 0;
		int middle = newHeightInPixel / 2;
		int bottom = newHeightInPixel;
		int left = 0;
		int center = newWidthInPixel / 2;
		int right = newWidthInPixel;

		// allocate the new instance
		ComputedRefPoint returnValue = new ComputedRefPoint(new Point2D(left,
				top), // topleft
				new Point2D(center, top), // topcenter
				new Point2D(right, top), // topright
				new Point2D(left, middle), // middleleft
				new Point2D(center, middle), // middlecenter
				new Point2D(right, middle), // middleright
				new Point2D(left, bottom), // bottomleft
				new Point2D(center, bottom), // bottomcenter
				new Point2D(right, bottom)); // bottomright

		return returnValue;
	}

	@Override
	public NodeLayoutComputer getShapeLayoutComputer() {
		// return itself as ashapeLayout Renderer
		return this;
	}

}
