package org.capcaval.lafabrique.application._test;

import java.io.IOException;

import org.capcaval.lafabrique.application.ApplicationTools;
import org.capcaval.lafabrique.common.CommandResult;
import org.capcaval.lafabrique.common.CommandResult.Type;
import org.capcaval.lafabrique.file.FileTools;
import org.capcaval.lafabrique.lang.SystemTools;
import org.junit.Assert;

public class GreeterApplicationTest {
	@org.junit.Test
	public void testErrorCommandNotFound(){
		CommandResult result = ApplicationTools.runApplication(GreeterApp.class, "unknownCommand", "noneParam");
		System.out.println("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
		System.out.println("testErrorCommandNotFound : \n" + result);
		System.out.println("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
		// the command shall not be found
		Assert.assertEquals(Type.ERROR, result.getType());
		// an human error message shall be available
		Assert.assertTrue( result.toString().contains("ERROR"));
	}

	@org.junit.Test
	public void testGreetWithParameter(){
		CommandResult result = ApplicationTools.runApplication(GreeterApp.class, "greet", "Louis");
		Assert.assertEquals("Hi Louis!", result.toString());
		
	}
	@org.junit.Test
	public void testGreetWithSeveralMethodSignature(){
		// here the greet without parameter should be called
		// and it returns Kajez name
		CommandResult result = ApplicationTools.runApplication(GreeterApp.class, "greet");
		Assert.assertEquals("Hi Jakez!", result.toString());
		
		// now the same method name with a parameter
		result = ApplicationTools.runApplication(GreeterApp.class, "greet", "Per");
		Assert.assertEquals("Hi Per!", result.toString());		

	}
	

	@org.junit.Test
	public void testGreetPersistence(){
		// set jojo as parameter on a persistent property
		String str = SystemTools.executeClassInAnotherJVM(GreeterApp.class, "hello", "jojo");
		Assert.assertEquals("Hello jojo!", str);

		// now jojo is saved inside the property file
		String propertiesStr = "";
		try {
			propertiesStr = FileTools.readStringfromFile("Greeter.properties");
			Assert.assertTrue(propertiesStr.contains("name=jojo"));
			
		} catch (IOException e) {
			e.printStackTrace();
		}

		// test without parameter the persistent property jojo shall be used
		str = SystemTools.executeClassInAnotherJVM(GreeterApp.class, "hello");
		Assert.assertEquals("Hello jojo!", str);
		

		System.out.println("finished");
	}

	
	@org.junit.Test
	public void testGreetPersistenceCustomFile(){
		// TODO ------------------------------------------
		// TODO add a dynamic setter of property file
		// TODO ------------------------------------------
		
		// set jojo as parameter on a persistent property
		String str = SystemTools.executeClassInAnotherJVM(GreeterApp.class, "hello", "juju");
		Assert.assertEquals("Hello juju!", str);

		// now jojo is saved inside the property file
		String propertiesStr = "";
		try {
			propertiesStr = FileTools.readStringfromFile("Greeter.properties");
			Assert.assertTrue(propertiesStr.contains("name=juju"));
			
		} catch (IOException e) {
			e.printStackTrace();
		}

		// test without parameter the persistent property jojo shall be used
		str = SystemTools.executeClassInAnotherJVM(GreeterApp.class, "hello");
		Assert.assertEquals("Hello juju!", str);
	}

	
	
	@org.junit.Test
	public void testGreetWithNoParameter(){
		// Make sure that the test is clean
		// Clean all parameters persistence
		ApplicationTools.cleanApplicationProperties(GreeterApp.class);
		
		Assert.assertTrue(FileTools.isFileExist("Greeter.properties")==false);
		
		CommandResult result = ApplicationTools.runApplication(GreeterApp.class, "hello");
		Assert.assertEquals("Hello unknown!", result.toString());
	}

	@org.junit.Test
	public void testSimpleAplicationTestHelp(){
		CommandResult result = ApplicationTools.runApplication(GreeterApp.class, "help");
		
		Assert.assertTrue(result.toString().contains("Version :"));
		Assert.assertTrue(result.toString().contains("About:"));
		
		System.out.println(result.toString());
	}
}
